@@ -205,17 +205,35 @@ class Node(ABC):
             # Use _node_id for internal node identifier (namespace separation)
             # This prevents collision with user's 'id' parameter
             self._node_id = kwargs.get("_node_id", self.__class__.__name__)
-            self.metadata = kwargs.get(
-                "metadata",
-                NodeMetadata(
+
+            # FIX: Use _node_metadata internally to avoid collision with user's "metadata" parameter
+            # This allows users to have parameters named "metadata" without conflicts
+            # Type-based routing: NodeMetadata object → internal, dict → user parameter
+            metadata_value = kwargs.get("metadata")
+            if isinstance(metadata_value, NodeMetadata):
+                # Core SDK or external code providing NodeMetadata object
+                self._node_metadata = metadata_value
+            elif metadata_value is None or not isinstance(metadata_value, dict):
+                # No metadata provided or invalid type → create default NodeMetadata
+                self._node_metadata = NodeMetadata(
                     id=self._node_id,  # NodeMetadata still uses 'id' internally
                     name=kwargs.get("name", self.__class__.__name__),
                     description=kwargs.get("description", self.__doc__ or ""),
                     version=kwargs.get("version", "1.0.0"),
                     author=kwargs.get("author", ""),
                     tags=kwargs.get("tags", set()),
-                ),
-            )
+                )
+            else:
+                # User provided dict as "metadata" parameter → create default NodeMetadata,
+                # and let the dict flow through to node.config (handled in filtering below)
+                self._node_metadata = NodeMetadata(
+                    id=self._node_id,
+                    name=kwargs.get("name", self.__class__.__name__),
+                    description=kwargs.get("description", self.__doc__ or ""),
+                    version=kwargs.get("version", "1.0.0"),
+                    author=kwargs.get("author", ""),
+                    tags=kwargs.get("tags", set()),
+                )
             self.logger = logging.getLogger(f"kailash.nodes.{self._node_id}")
 
             # Filter out internal fields from config with comprehensive parameter handling
@@ -234,7 +252,9 @@ class Node(ABC):
 
             # Comprehensive parameter filtering: handle ALL potential conflicts
             # Fields that are always internal (never user parameters)
-            always_internal = {"metadata", "_node_id"}
+            # FIX: Removed "metadata" from this set to allow users to use "metadata" as parameter name
+            # The internal NodeMetadata is now stored in self._node_metadata
+            always_internal = {"_node_id"}
 
             # Fields that can be either internal or user parameters
             # Note: 'id' removed from this list - users can now use 'id' freely
@@ -267,6 +287,11 @@ class Node(ABC):
                 # Check for other internal patterns
                 if field_name.startswith("_"):  # Private fields
                     return True
+                # FIX: Filter out NodeMetadata objects (they're internal, not user parameters)
+                if field_name == "metadata" and isinstance(
+                    kwargs.get(field_name), NodeMetadata
+                ):
+                    return True
                 return False
 
             # Apply comprehensive filtering
@@ -373,6 +398,64 @@ class Node(ABC):
         """
         self._node_id = value
 
+    @property
+    def metadata(self) -> NodeMetadata:
+        """
+        Backward compatibility property for node metadata.
+
+        Returns the node's internal NodeMetadata object (_node_metadata).
+        This property maintains backward compatibility for code that accesses node.metadata.
+
+        The internal metadata is now _node_metadata to prevent namespace collision
+        with user's 'metadata' parameter.
+
+        Returns:
+            NodeMetadata object containing node identification and documentation
+
+        Note:
+            Users can now have parameters named "metadata" without conflicts.
+            The parameter will be in node.config['metadata'], while this property
+            returns the internal NodeMetadata object.
+        """
+        return self._node_metadata
+
+    @metadata.setter
+    def metadata(self, value: NodeMetadata | dict):
+        """
+        Setter for backward compatibility with type-based routing.
+
+        This setter routes the value based on its type:
+        - NodeMetadata object → sets internal _node_metadata (Core SDK usage)
+        - dict → sets node.config['metadata'] (user parameter)
+        - other types → raises TypeError for safety
+
+        Args:
+            value: NodeMetadata object or dict
+
+        Raises:
+            TypeError: If value is neither NodeMetadata nor dict
+
+        Example:
+            >>> # Core SDK usage
+            >>> node.metadata = NodeMetadata(name="custom")  # Routes to _node_metadata
+
+            >>> # User parameter (if node has "metadata" parameter)
+            >>> node.metadata = {"key": "value"}  # Routes to config['metadata']
+        """
+        if isinstance(value, NodeMetadata):
+            # Core SDK usage - route to internal metadata
+            self._node_metadata = value
+        elif isinstance(value, dict) or value is None:
+            # User parameter - route to config (None is valid for optional parameters)
+            self.config["metadata"] = value
+        else:
+            # Ambiguous type - raise error for safety
+            raise TypeError(
+                f"metadata must be NodeMetadata, dict, or None, got {type(value).__name__}. "
+                f"For Core SDK metadata, use NodeMetadata object. "
+                f"For user metadata parameter, use dict or None."
+            )
+
     @abstractmethod
     def get_parameters(self) -> dict[str, NodeParameter]:
         """Define the parameters this node accepts.
@@ -588,6 +671,9 @@ class Node(ABC):
                 # Skip validation for template expressions like ${variable_name}
                 if isinstance(value, str) and self._is_template_expression(value):
                     continue
+                # FIX: Allow None for optional parameters (not required)
+                if value is None and not param_def.required:
+                    continue
                 if not isinstance(value, param_def.type):
                     try:
                         # Special handling for datetime conversion from ISO strings
@@ -1235,6 +1321,14 @@ class Node(ABC):
             # Merge runtime inputs with config (runtime inputs take precedence)
             merged_inputs = {**self.config, **runtime_inputs}
 
+            # Resolve ${param} templates in merged parameters (v0.9.30)
+            # This enables dynamic parameter injection in nested configurations
+            # Example: {"filter": {"tag": "${tag}"}} with runtime_inputs={"tag": "local"}
+            # Becomes: {"filter": {"tag": "local"}}
+            from kailash.runtime.template_resolver import resolve_templates
+
+            merged_inputs = resolve_templates(merged_inputs, runtime_inputs)
+
             # Handle nested config case (for nodes that store parameters in config['config'])
             if "config" in merged_inputs and isinstance(merged_inputs["config"], dict):
                 # Extract nested config but preserve runtime input precedence
@@ -2196,6 +2290,74 @@ class NodeRegistry:
         """
         return cls._nodes.copy()
 
+    @classmethod
+    def unregister(cls, node_name: str) -> bool:
+        """Unregister a single node from the registry.
+
+        Removes a specific node class from the global registry. Used for:
+
+        1. Test isolation - Clean up test-specific nodes
+        2. Dynamic reloading - Remove before re-registering
+        3. Instance cleanup - Remove DataFlow instance nodes
+
+        Args:
+            node_name: Name of the node to unregister
+
+        Returns:
+            bool: True if node was unregistered, False if not found
+
+        Side effects:
+            - Removes node from _nodes dictionary
+            - Logs the unregistration
+            - Existing node instances remain valid
+
+        Example usage:
+            NodeRegistry.unregister('UserCreateNode')
+            NodeRegistry.unregister('custom_alias')
+
+        Used by:
+            - Test cleanup fixtures
+            - DataFlow instance cleanup
+            - Dynamic node reloading
+        """
+        if node_name in cls._nodes:
+            del cls._nodes[node_name]
+            logging.debug(f"Unregistered node '{node_name}'")
+            return True
+        return False
+
+    @classmethod
+    def unregister_nodes(cls, node_names: list[str]) -> int:
+        """Unregister multiple nodes from the registry.
+
+        Batch unregistration for cleaning up a set of related nodes.
+        Returns count of successfully unregistered nodes.
+
+        Args:
+            node_names: List of node names to unregister
+
+        Returns:
+            int: Number of nodes successfully unregistered
+
+        Example usage:
+            # Clean up DataFlow model nodes
+            nodes = ['UserCreateNode', 'UserReadNode', 'UserUpdateNode']
+            count = NodeRegistry.unregister_nodes(nodes)
+            print(f"Unregistered {count} nodes")
+
+        Used by:
+            - DataFlow.cleanup_nodes()
+            - Test fixture teardown
+            - Multi-instance cleanup
+        """
+        count = 0
+        for node_name in node_names:
+            if cls.unregister(node_name):
+                count += 1
+        if count > 0:
+            logging.info(f"Unregistered {count} nodes from registry")
+        return count
+
     @classmethod
     def clear(cls):
         """Clear all registered nodes.
