@@ -24,6 +24,7 @@ from datetime import UTC, datetime
 from typing import Any, Dict, List, Optional, Set, Tuple, Union
 
 import networkx as nx
+
 from kailash.nodes.base import Node
 from kailash.nodes.base_async import AsyncNode
 from kailash.resources import ResourceRegistry
@@ -296,12 +297,29 @@ class AsyncLocalRuntime(LocalRuntime):
     """
     Async-optimized runtime for Kailash workflows.
 
-    Extends LocalRuntime with advanced async execution capabilities:
-    - Concurrent node execution where possible
-    - Integrated ResourceRegistry support
-    - Workflow analysis and optimization
-    - Advanced performance tracking
-    - Circuit breaker patterns
+    Extends LocalRuntime with advanced async execution capabilities while
+    inheriting all enterprise features through shared mixin architecture.
+
+    Inherits from:
+        LocalRuntime: Provides 100% feature parity with sync runtime
+            ├─ BaseRuntime: Core runtime foundation and configuration
+            ├─ CycleExecutionMixin: Cyclic workflow execution delegation
+            ├─ ValidationMixin: Workflow validation and contract checking
+            └─ ConditionalExecutionMixin: Conditional execution and branching logic
+
+    Async-Specific Extensions:
+        - WorkflowAnalyzer: Analyzes workflows for optimization opportunities
+        - ExecutionContext: Async context with integrated resource access
+        - Level-based parallel execution: Executes independent nodes concurrently
+        - Semaphore-based concurrency control: Limits concurrent node execution
+        - Thread pool for sync nodes: Executes sync nodes without blocking async loop
+        - Advanced performance tracking: Detailed metrics collection
+
+    Execution Strategies:
+        The runtime automatically selects the optimal execution strategy:
+        - Pure async: All nodes are async (fastest, full concurrency)
+        - Mixed: Combination of sync and async nodes (balanced)
+        - Sync in thread pool: All sync nodes (compatibility mode)
 
     Example:
         ```python
@@ -363,13 +381,31 @@ class AsyncLocalRuntime(LocalRuntime):
         # Thread pool for sync node execution
         self.thread_pool = ThreadPoolExecutor(max_workers=thread_pool_size)
 
-        # Execution semaphore for concurrency control
-        self.execution_semaphore = asyncio.Semaphore(max_concurrent_nodes)
+        # P0-7 FIX: Don't create event loop or semaphore in __init__
+        # Will be lazily initialized during execute_workflow_async() execution
+        # This prevents race conditions where __init__ runs outside async context
+        self._semaphore = None
+        self._max_concurrent = max_concurrent_nodes
 
         logger.info(
             f"AsyncLocalRuntime initialized with max_concurrent_nodes={max_concurrent_nodes}"
         )
 
+    @property
+    def execution_semaphore(self) -> asyncio.Semaphore:
+        """
+        Lazily create execution semaphore when accessed.
+
+        P0-7 FIX: Semaphore must be created in async context (with running event loop).
+        Creating in __init__ causes race conditions in FastAPI/Docker deployments.
+        """
+        if self._semaphore is None:
+            self._semaphore = asyncio.Semaphore(self._max_concurrent)
+            logger.debug(
+                f"Execution semaphore created with limit={self._max_concurrent}"
+            )
+        return self._semaphore
+
     def execute(
         self,
         workflow,
@@ -455,7 +491,7 @@ class AsyncLocalRuntime(LocalRuntime):
         workflow,
         inputs: Dict[str, Any],
         context: Optional[ExecutionContext] = None,
-    ) -> Dict[str, Any]:
+    ) -> Tuple[Dict[str, Any], str]:
         """
         Execute workflow with native async support.
 
@@ -471,13 +507,18 @@ class AsyncLocalRuntime(LocalRuntime):
             context: Optional execution context
 
         Returns:
-            Dictionary containing execution results and metrics
+            Tuple of (results dict, run_id) - SAME AS LocalRuntime.execute()
+            - results: Dictionary mapping node_id -> node output
+            - run_id: Unique execution identifier
 
         Raises:
             WorkflowExecutionError: If execution fails
         """
         start_time = time.time()
 
+        # Generate run_id for tracking (consistent with LocalRuntime)
+        run_id = f"run_{int(time.time() * 1000)}"
+
         # Create execution context
         if context is None:
             context = ExecutionContext(resource_registry=self.resource_registry)
@@ -486,26 +527,47 @@ class AsyncLocalRuntime(LocalRuntime):
         context.variables.update(inputs)
 
         try:
-            # Analyze workflow if enabled
-            execution_plan = None
-            if self.analyzer:
-                execution_plan = self.analyzer.analyze(workflow)
+            # Check for conditional workflow with skip_branches mode
+            # Only use conditional execution approach if skip_branches is enabled
+            if (
+                self._has_conditional_patterns(workflow)
+                and self.conditional_execution == "skip_branches"
+            ):
                 logger.info(
-                    f"Execution plan: {execution_plan.max_concurrent_nodes} max concurrent, "
-                    f"{len(execution_plan.execution_levels)} levels"
+                    "Conditional workflow with skip_branches mode detected, using conditional execution"
                 )
-
-            # Choose execution strategy based on analysis
-            if execution_plan and execution_plan.is_fully_async:
-                result = await self._execute_fully_async_workflow(
-                    workflow, context, execution_plan
-                )
-            elif execution_plan and execution_plan.has_async_nodes:
-                result = await self._execute_mixed_workflow(
-                    workflow, context, execution_plan
+                # Use inherited conditional execution from ConditionalExecutionMixin
+                tracker_result = await self._execute_conditional_approach(
+                    workflow=workflow,
+                    parameters=inputs,
+                    task_manager=None,
+                    run_id=run_id,
+                    workflow_context=None,
                 )
             else:
-                result = await self._execute_sync_workflow(workflow, context)
+                # Regular execution path
+                # Analyze workflow if enabled
+                execution_plan = None
+                if self.analyzer:
+                    execution_plan = self.analyzer.analyze(workflow)
+                    logger.info(
+                        f"Execution plan: {execution_plan.max_concurrent_nodes} max concurrent, "
+                        f"{len(execution_plan.execution_levels)} levels"
+                    )
+
+                # Choose execution strategy based on analysis
+                if execution_plan and execution_plan.is_fully_async:
+                    tracker_result = await self._execute_fully_async_workflow(
+                        workflow, context, execution_plan
+                    )
+                elif execution_plan and execution_plan.has_async_nodes:
+                    tracker_result = await self._execute_mixed_workflow(
+                        workflow, context, execution_plan
+                    )
+                else:
+                    tracker_result = await self._execute_sync_workflow(
+                        workflow, context
+                    )
 
             # Update total execution time
             total_time = time.time() - start_time
@@ -513,7 +575,24 @@ class AsyncLocalRuntime(LocalRuntime):
 
             logger.info(f"Workflow execution completed in {total_time:.2f}s")
 
-            return result
+            # Extract plain results dict (consistent with LocalRuntime)
+            # Conditional approach (skip_branches mode) returns plain dict, other methods return tracker wrapper
+            if (
+                self._has_conditional_patterns(workflow)
+                and self.conditional_execution == "skip_branches"
+            ):
+                results = (
+                    tracker_result  # Already plain dict from conditional execution
+                )
+            else:
+                results = (
+                    tracker_result.get("results", {})
+                    if isinstance(tracker_result, dict)
+                    else tracker_result
+                )
+
+            # Return same structure as LocalRuntime.execute(): (results, run_id)
+            return results, run_id
 
         except Exception as e:
             logger.error(f"Workflow execution failed: {e}")
@@ -638,6 +717,19 @@ class AsyncLocalRuntime(LocalRuntime):
                 workflow, node_id, node_outputs, inputs
             )
 
+            # CONDITIONAL EXECUTION: Skip nodes that only receive None inputs from conditional routing
+            # Uses shared mixin method (ConditionalExecutionMixin._should_skip_conditional_node)
+            # Pass results dict for transitive dependency checking
+            if self._should_skip_conditional_node(
+                workflow, node_id, node_inputs, results
+            ):
+                logger.info(
+                    f"Skipping node {node_id} - all conditional inputs are None"
+                )
+                results[node_id] = None
+                node_outputs[node_id] = None
+                continue
+
             # Execute node
             try:
                 result = node_instance.execute(**node_inputs)
@@ -741,6 +833,18 @@ class AsyncLocalRuntime(LocalRuntime):
                     workflow, node_id, tracker, context
                 )
 
+                # CONDITIONAL EXECUTION: Skip nodes that only receive None inputs from conditional routing
+                # Uses shared mixin method (ConditionalExecutionMixin._should_skip_conditional_node)
+                # Pass tracker.results for transitive dependency checking
+                if self._should_skip_conditional_node(
+                    workflow, node_id, inputs, tracker.results
+                ):
+                    logger.info(
+                        f"Skipping node {node_id} - all conditional inputs are None"
+                    )
+                    await tracker.record_result(node_id, None, 0.0)
+                    return
+
                 # Execute async node
                 if isinstance(node_instance, AsyncNode):
                     # Add resource registry to inputs if available
@@ -795,6 +899,18 @@ class AsyncLocalRuntime(LocalRuntime):
                     workflow, node_id, tracker, context
                 )
 
+                # CONDITIONAL EXECUTION: Skip nodes that only receive None inputs from conditional routing
+                # Uses shared mixin method (ConditionalExecutionMixin._should_skip_conditional_node)
+                # Pass tracker.results for transitive dependency checking
+                if self._should_skip_conditional_node(
+                    workflow, node_id, inputs, tracker.results
+                ):
+                    logger.info(
+                        f"Skipping node {node_id} - all conditional inputs are None"
+                    )
+                    await tracker.record_result(node_id, None, 0.0)
+                    return
+
                 # Execute sync node in thread pool
                 result = await self._execute_sync_node_in_thread(node_instance, inputs)
 
@@ -924,16 +1040,59 @@ class AsyncLocalRuntime(LocalRuntime):
         return inputs
 
     async def cleanup(self) -> None:
-        """Clean up runtime resources."""
-        logger.info("Cleaning up AsyncLocalRuntime")
+        """
+        Clean up runtime resources (idempotent).
 
-        # Clean up thread pool
-        self.thread_pool.shutdown(wait=True)
+        P0-8 FIX: Enhanced cleanup with proper resource management.
+        Safe to call multiple times - tracks cleanup state.
 
-        # Clean up resource registry if owned
-        if self.resource_registry:
-            await self.resource_registry.cleanup()
+        Recommended usage with FastAPI lifespan:
+            ```python
+            from contextlib import asynccontextmanager
+            from fastapi import FastAPI
 
+            @asynccontextmanager
+            async def lifespan(app: FastAPI):
+                # Startup
+                runtime = AsyncLocalRuntime()
+                yield {"runtime": runtime}
+                # Shutdown
+                await runtime.cleanup()
+
+            app = FastAPI(lifespan=lifespan)
+            ```
+        """
+        # Track cleanup to make it idempotent
+        if hasattr(self, "_cleaned_up") and self._cleaned_up:
+            logger.debug("AsyncLocalRuntime already cleaned up, skipping")
+            return
+
+        logger.info("Cleaning up AsyncLocalRuntime resources...")
+
+        # Clean up thread pool (if exists and not already shutdown)
+        if hasattr(self, "thread_pool") and self.thread_pool:
+            try:
+                self.thread_pool.shutdown(wait=True)
+                logger.debug("Thread pool shutdown successfully")
+            except Exception as e:
+                logger.warning(f"Error shutting down thread pool: {e}")
+            finally:
+                self.thread_pool = None
+
+        # Clean up resource registry (if owned)
+        if hasattr(self, "resource_registry") and self.resource_registry:
+            try:
+                await self.resource_registry.cleanup()
+                logger.debug("Resource registry cleaned up")
+            except Exception as e:
+                logger.warning(f"Error cleaning up resource registry: {e}")
+
+        # Clean up semaphore reference
+        if hasattr(self, "_semaphore"):
+            self._semaphore = None
+
+        # Mark as cleaned up
+        self._cleaned_up = True
         logger.info("AsyncLocalRuntime cleanup complete")
 
     def __del__(self):
